# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .cost_data import CostData
from .requests_data import RequestsData

__all__ = ["PagedBudgetList", "Item", "ItemTotals", "ItemTotalsBudgetTransactions"]


class ItemTotalsBudgetTransactions(BaseModel):
    blocked: int

    blocked_external: int

    exceeded: int

    successful: int

    error: Optional[int] = None

    total: Optional[int] = None


class ItemTotals(BaseModel):
    cost: CostData

    requests: RequestsData

    budget_transactions: Optional[ItemTotalsBudgetTransactions] = None


class Item(BaseModel):
    base_cost_estimate: Literal["max"]

    budget_creation_timestamp: datetime

    budget_id: str

    budget_name: str

    budget_response_type: Literal["block", "allow"]

    budget_type: Literal["conservative", "liberal"]

    budget_update_timestamp: datetime

    currency: Literal["usd"]

    max: float

    totals: ItemTotals

    budget_tags: Optional[List[str]] = None


class PagedBudgetList(BaseModel):
    current_page: Optional[int] = FieldInfo(alias="currentPage", default=None)

    has_next_page: Optional[bool] = FieldInfo(alias="hasNextPage", default=None)

    has_previous_page: Optional[bool] = FieldInfo(alias="hasPreviousPage", default=None)

    items: Optional[List[Item]] = None

    message: Optional[str] = None

    page_size: Optional[int] = FieldInfo(alias="pageSize", default=None)

    request_id: Optional[str] = None

    total_count: Optional[int] = FieldInfo(alias="totalCount", default=None)

    total_pages: Optional[int] = FieldInfo(alias="totalPages", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tags import (
    TagsResource,
    AsyncTagsResource,
    TagsResourceWithRawResponse,
    AsyncTagsResourceWithRawResponse,
    TagsResourceWithStreamingResponse,
    AsyncTagsResourceWithStreamingResponse,
)
from .budgets import (
    BudgetsResource,
    AsyncBudgetsResource,
    BudgetsResourceWithRawResponse,
    AsyncBudgetsResourceWithRawResponse,
    BudgetsResourceWithStreamingResponse,
    AsyncBudgetsResourceWithStreamingResponse,
)

__all__ = [
    "TagsResource",
    "AsyncTagsResource",
    "TagsResourceWithRawResponse",
    "AsyncTagsResourceWithRawResponse",
    "TagsResourceWithStreamingResponse",
    "AsyncTagsResourceWithStreamingResponse",
    "BudgetsResource",
    "AsyncBudgetsResource",
    "BudgetsResourceWithRawResponse",
    "AsyncBudgetsResourceWithRawResponse",
    "BudgetsResourceWithStreamingResponse",
    "AsyncBudgetsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .cost_data import CostData
from .requests_data import RequestsData

__all__ = ["BudgetHistoryResponse", "BudgetHistory", "BudgetHistoryTotals", "BudgetHistoryTotalsBudgetTransactions"]


class BudgetHistoryTotalsBudgetTransactions(BaseModel):
    blocked: int

    blocked_external: int

    exceeded: int

    successful: int

    error: Optional[int] = None

    total: Optional[int] = None


class BudgetHistoryTotals(BaseModel):
    cost: CostData

    requests: RequestsData

    budget_transactions: Optional[BudgetHistoryTotalsBudgetTransactions] = None


class BudgetHistory(BaseModel):
    budget_name: Optional[str] = None

    base_cost_estimate: Optional[Literal["max"]] = None

    budget_id: Optional[str] = None

    budget_reset_timestamp: Optional[datetime] = None

    budget_response_type: Optional[Literal["block", "allow"]] = None

    budget_tags: Optional[List[str]] = None

    budget_type: Optional[Literal["conservative", "liberal"]] = None

    max: Optional[float] = None

    totals: Optional[BudgetHistoryTotals] = None


class BudgetHistoryResponse(BaseModel):
    budget_history: BudgetHistory

    request_id: str

    message: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from payi import Payi, AsyncPayi
from tests.utils import assert_matches_type
from payi.types.budgets import (
    TagListResponse,
    TagCreateResponse,
    TagDeleteResponse,
    TagRemoveResponse,
    TagUpdateResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTags:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Payi) -> None:
        tag = client.budgets.tags.create(
            budget_id="budget_id",
            budget_tags=["tag1", "tag2"],
        )
        assert_matches_type(TagCreateResponse, tag, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Payi) -> None:
        response = client.budgets.tags.with_raw_response.create(
            budget_id="budget_id",
            budget_tags=["tag1", "tag2"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tag = response.parse()
        assert_matches_type(TagCreateResponse, tag, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Payi) -> None:
        with client.budgets.tags.with_streaming_response.create(
            budget_id="budget_id",
            budget_tags=["tag1", "tag2"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tag = response.parse()
            assert_matches_type(TagCreateResponse, tag, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: Payi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `budget_id` but received ''"):
            client.budgets.tags.with_raw_response.create(
                budget_id="",
                budget_tags=["tag1", "tag2"],
            )

    @parametrize
    def test_method_update(self, client: Payi) -> None:
        tag = client.budgets.tags.update(
            budget_id="budget_id",
            budget_tags=["tag1", "tag2"],
        )
        assert_matches_type(TagUpdateResponse, tag, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Payi) -> None:
        response = client.budgets.tags.with_raw_response.update(
            budget_id="budget_id",
            budget_tags=["tag1", "tag2"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tag = response.parse()
        assert_matches_type(TagUpdateResponse, tag, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Payi) -> None:
        with client.budgets.tags.with_streaming_response.update(
            budget_id="budget_id",
            budget_tags=["tag1", "tag2"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tag = response.parse()
            assert_matches_type(TagUpdateResponse, tag, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Payi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `budget_id` but received ''"):
            client.budgets.tags.with_raw_response.update(
                budget_id="",
                budget_tags=["tag1", "tag2"],
            )

    @parametrize
    def test_method_list(self, client: Payi) -> None:
        tag = client.budgets.tags.list(
            "budget_id",
        )
        assert_matches_type(TagListResponse, tag, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Payi) -> None:
        response = client.budgets.tags.with_raw_response.list(
            "budget_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tag = response.parse()
        assert_matches_type(TagListResponse, tag, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Payi) -> None:
        with client.budgets.tags.with_streaming_response.list(
            "budget_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tag = response.parse()
            assert_matches_type(TagListResponse, tag, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: Payi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `budget_id` but received ''"):
            client.budgets.tags.with_raw_response.list(
                "",
            )

    @parametrize
    def test_method_delete(self, client: Payi) -> None:
        tag = client.budgets.tags.delete(
            "budget_id",
        )
        assert_matches_type(TagDeleteResponse, tag, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: Payi) -> None:
        response = client.budgets.tags.with_raw_response.delete(
            "budget_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tag = response.parse()
        assert_matches_type(TagDeleteResponse, tag, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: Payi) -> None:
        with client.budgets.tags.with_streaming_response.delete(
            "budget_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tag = response.parse()
            assert_matches_type(TagDeleteResponse, tag, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Payi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `budget_id` but received ''"):
            client.budgets.tags.with_raw_response.delete(
                "",
            )

    @parametrize
    def test_method_remove(self, client: Payi) -> None:
        tag = client.budgets.tags.remove(
            budget_id="budget_id",
            budget_tags=["tag1", "tag2"],
        )
        assert_matches_type(TagRemoveResponse, tag, path=["response"])

    @parametrize
    def test_raw_response_remove(self, client: Payi) -> None:
        response = client.budgets.tags.with_raw_response.remove(
            budget_id="budget_id",
            budget_tags=["tag1", "tag2"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tag = response.parse()
        assert_matches_type(TagRemoveResponse, tag, path=["response"])

    @parametrize
    def test_streaming_response_remove(self, client: Payi) -> None:
        with client.budgets.tags.with_streaming_response.remove(
            budget_id="budget_id",
            budget_tags=["tag1", "tag2"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tag = response.parse()
            assert_matches_type(TagRemoveResponse, tag, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_remove(self, client: Payi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `budget_id` but received ''"):
            client.budgets.tags.with_raw_response.remove(
                budget_id="",
                budget_tags=["tag1", "tag2"],
            )


class TestAsyncTags:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncPayi) -> None:
        tag = await async_client.budgets.tags.create(
            budget_id="budget_id",
            budget_tags=["tag1", "tag2"],
        )
        assert_matches_type(TagCreateResponse, tag, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncPayi) -> None:
        response = await async_client.budgets.tags.with_raw_response.create(
            budget_id="budget_id",
            budget_tags=["tag1", "tag2"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tag = await response.parse()
        assert_matches_type(TagCreateResponse, tag, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncPayi) -> None:
        async with async_client.budgets.tags.with_streaming_response.create(
            budget_id="budget_id",
            budget_tags=["tag1", "tag2"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tag = await response.parse()
            assert_matches_type(TagCreateResponse, tag, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncPayi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `budget_id` but received ''"):
            await async_client.budgets.tags.with_raw_response.create(
                budget_id="",
                budget_tags=["tag1", "tag2"],
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncPayi) -> None:
        tag = await async_client.budgets.tags.update(
            budget_id="budget_id",
            budget_tags=["tag1", "tag2"],
        )
        assert_matches_type(TagUpdateResponse, tag, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncPayi) -> None:
        response = await async_client.budgets.tags.with_raw_response.update(
            budget_id="budget_id",
            budget_tags=["tag1", "tag2"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tag = await response.parse()
        assert_matches_type(TagUpdateResponse, tag, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncPayi) -> None:
        async with async_client.budgets.tags.with_streaming_response.update(
            budget_id="budget_id",
            budget_tags=["tag1", "tag2"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tag = await response.parse()
            assert_matches_type(TagUpdateResponse, tag, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncPayi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `budget_id` but received ''"):
            await async_client.budgets.tags.with_raw_response.update(
                budget_id="",
                budget_tags=["tag1", "tag2"],
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncPayi) -> None:
        tag = await async_client.budgets.tags.list(
            "budget_id",
        )
        assert_matches_type(TagListResponse, tag, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncPayi) -> None:
        response = await async_client.budgets.tags.with_raw_response.list(
            "budget_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tag = await response.parse()
        assert_matches_type(TagListResponse, tag, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncPayi) -> None:
        async with async_client.budgets.tags.with_streaming_response.list(
            "budget_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tag = await response.parse()
            assert_matches_type(TagListResponse, tag, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncPayi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `budget_id` but received ''"):
            await async_client.budgets.tags.with_raw_response.list(
                "",
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncPayi) -> None:
        tag = await async_client.budgets.tags.delete(
            "budget_id",
        )
        assert_matches_type(TagDeleteResponse, tag, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncPayi) -> None:
        response = await async_client.budgets.tags.with_raw_response.delete(
            "budget_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tag = await response.parse()
        assert_matches_type(TagDeleteResponse, tag, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncPayi) -> None:
        async with async_client.budgets.tags.with_streaming_response.delete(
            "budget_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tag = await response.parse()
            assert_matches_type(TagDeleteResponse, tag, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncPayi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `budget_id` but received ''"):
            await async_client.budgets.tags.with_raw_response.delete(
                "",
            )

    @parametrize
    async def test_method_remove(self, async_client: AsyncPayi) -> None:
        tag = await async_client.budgets.tags.remove(
            budget_id="budget_id",
            budget_tags=["tag1", "tag2"],
        )
        assert_matches_type(TagRemoveResponse, tag, path=["response"])

    @parametrize
    async def test_raw_response_remove(self, async_client: AsyncPayi) -> None:
        response = await async_client.budgets.tags.with_raw_response.remove(
            budget_id="budget_id",
            budget_tags=["tag1", "tag2"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tag = await response.parse()
        assert_matches_type(TagRemoveResponse, tag, path=["response"])

    @parametrize
    async def test_streaming_response_remove(self, async_client: AsyncPayi) -> None:
        async with async_client.budgets.tags.with_streaming_response.remove(
            budget_id="budget_id",
            budget_tags=["tag1", "tag2"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tag = await response.parse()
            assert_matches_type(TagRemoveResponse, tag, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_remove(self, async_client: AsyncPayi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `budget_id` but received ''"):
            await async_client.budgets.tags.with_raw_response.remove(
                budget_id="",
                budget_tags=["tag1", "tag2"],
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .cost_details import CostDetails

__all__ = ["ProxyResult", "Budgets", "Cost"]


class Budgets(BaseModel):
    state: Optional[Literal["ok", "blocked", "blocked_external", "exceeded", "failed"]] = None


class Cost(BaseModel):
    currency: Optional[Literal["usd"]] = None

    input: Optional[CostDetails] = None

    output: Optional[CostDetails] = None

    total: Optional[CostDetails] = None


class ProxyResult(BaseModel):
    budgets: Optional[Dict[str, Budgets]] = None

    cost: Optional[Cost] = None

    request_id: Optional[str] = None

    request_tags: Optional[List[str]] = None



import json
from typing import List, cast
from typing_extensions import Annotated

import httpx
import pytest
import pydantic

from payi import Payi, AsyncPayi, BaseModel
from payi._response import (
    APIResponse,
    BaseAPIResponse,
    AsyncAPIResponse,
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    extract_response_type,
)
from payi._streaming import Stream
from payi._base_client import FinalRequestOptions


class ConcreteBaseAPIResponse(APIResponse[bytes]):
    ...


class ConcreteAPIResponse(APIResponse[List[str]]):
    ...


class ConcreteAsyncAPIResponse(APIResponse[httpx.Response]):
    ...


def test_extract_response_type_direct_classes() -> None:
    assert extract_response_type(BaseAPIResponse[str]) == str
    assert extract_response_type(APIResponse[str]) == str
    assert extract_response_type(AsyncAPIResponse[str]) == str


def test_extract_response_type_direct_class_missing_type_arg() -> None:
    with pytest.raises(
        RuntimeError,
        match="Expected type <class 'payi._response.AsyncAPIResponse'> to have a type argument at index 0 but it did not",
    ):
        extract_response_type(AsyncAPIResponse)


def test_extract_response_type_concrete_subclasses() -> None:
    assert extract_response_type(ConcreteBaseAPIResponse) == bytes
    assert extract_response_type(ConcreteAPIResponse) == List[str]
    assert extract_response_type(ConcreteAsyncAPIResponse) == httpx.Response


def test_extract_response_type_binary_response() -> None:
    assert extract_response_type(BinaryAPIResponse) == bytes
    assert extract_response_type(AsyncBinaryAPIResponse) == bytes


class PydanticModel(pydantic.BaseModel):
    ...


def test_response_parse_mismatched_basemodel(client: Payi) -> None:
    response = APIResponse(
        raw=httpx.Response(200, content=b"foo"),
        client=client,
        stream=False,
        stream_cls=None,
        cast_to=str,
        options=FinalRequestOptions.construct(method="get", url="/foo"),
    )

    with pytest.raises(
        TypeError,
        match="Pydantic models must subclass our base model type, e.g. `from payi import BaseModel`",
    ):
        response.parse(to=PydanticModel)


@pytest.mark.asyncio
async def test_async_response_parse_mismatched_basemodel(async_client: AsyncPayi) -> None:
    response = AsyncAPIResponse(
        raw=httpx.Response(200, content=b"foo"),
        client=async_client,
        stream=False,
        stream_cls=None,
        cast_to=str,
        options=FinalRequestOptions.construct(method="get", url="/foo"),
    )

    with pytest.raises(
        TypeError,
        match="Pydantic models must subclass our base model type, e.g. `from payi import BaseModel`",
    ):
        await response.parse(to=PydanticModel)


def test_response_parse_custom_stream(client: Payi) -> None:
    response = APIResponse(
        raw=httpx.Response(200, content=b"foo"),
        client=client,
        stream=True,
        stream_cls=None,
        cast_to=str,
        options=FinalRequestOptions.construct(method="get", url="/foo"),
    )

    stream = response.parse(to=Stream[int])
    assert stream._cast_to == int


@pytest.mark.asyncio
async def test_async_response_parse_custom_stream(async_client: AsyncPayi) -> None:
    response = AsyncAPIResponse(
        raw=httpx.Response(200, content=b"foo"),
        client=async_client,
        stream=True,
        stream_cls=None,
        cast_to=str,
        options=FinalRequestOptions.construct(method="get", url="/foo"),
    )

    stream = await response.parse(to=Stream[int])
    assert stream._cast_to == int


class CustomModel(BaseModel):
    foo: str
    bar: int


def test_response_parse_custom_model(client: Payi) -> None:
    response = APIResponse(
        raw=httpx.Response(200, content=json.dumps({"foo": "hello!", "bar": 2})),
        client=client,
        stream=False,
        stream_cls=None,
        cast_to=str,
        options=FinalRequestOptions.construct(method="get", url="/foo"),
    )

    obj = response.parse(to=CustomModel)
    assert obj.foo == "hello!"
    assert obj.bar == 2


@pytest.mark.asyncio
async def test_async_response_parse_custom_model(async_client: AsyncPayi) -> None:
    response = AsyncAPIResponse(
        raw=httpx.Response(200, content=json.dumps({"foo": "hello!", "bar": 2})),
        client=async_client,
        stream=False,
        stream_cls=None,
        cast_to=str,
        options=FinalRequestOptions.construct(method="get", url="/foo"),
    )

    obj = await response.parse(to=CustomModel)
    assert obj.foo == "hello!"
    assert obj.bar == 2


def test_response_parse_annotated_type(client: Payi) -> None:
    response = APIResponse(
        raw=httpx.Response(200, content=json.dumps({"foo": "hello!", "bar": 2})),
        client=client,
        stream=False,
        stream_cls=None,
        cast_to=str,
        options=FinalRequestOptions.construct(method="get", url="/foo"),
    )

    obj = response.parse(
        to=cast("type[CustomModel]", Annotated[CustomModel, "random metadata"]),
    )
    assert obj.foo == "hello!"
    assert obj.bar == 2


async def test_async_response_parse_annotated_type(async_client: AsyncPayi) -> None:
    response = AsyncAPIResponse(
        raw=httpx.Response(200, content=json.dumps({"foo": "hello!", "bar": 2})),
        client=async_client,
        stream=False,
        stream_cls=None,
        cast_to=str,
        options=FinalRequestOptions.construct(method="get", url="/foo"),
    )

    obj = await response.parse(
        to=cast("type[CustomModel]", Annotated[CustomModel, "random metadata"]),
    )
    assert obj.foo == "hello!"
    assert obj.bar == 2



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .category_resource_response import CategoryResourceResponse

__all__ = ["CategoryDeleteResourceResponse"]

CategoryDeleteResourceResponse = List[CategoryResourceResponse]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .budget_tags import BudgetTags

__all__ = ["TagDeleteResponse"]

TagDeleteResponse = List[BudgetTags]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from payi import Payi, AsyncPayi
from payi.types import ProxyResult
from payi._utils import parse_datetime
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestIngest:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_units(self, client: Payi) -> None:
        ingest = client.ingest.units(
            category="x",
            input=0,
            output=0,
            resource="x",
        )
        assert_matches_type(ProxyResult, ingest, path=["response"])

    @parametrize
    def test_method_units_with_all_params(self, client: Payi) -> None:
        ingest = client.ingest.units(
            category="x",
            input=0,
            output=0,
            resource="x",
            event_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
            budget_ids=["budgetId1", "budgetId_2"],
            request_tags=["requestTag1", "request_tag_2"],
        )
        assert_matches_type(ProxyResult, ingest, path=["response"])

    @parametrize
    def test_raw_response_units(self, client: Payi) -> None:
        response = client.ingest.with_raw_response.units(
            category="x",
            input=0,
            output=0,
            resource="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ingest = response.parse()
        assert_matches_type(ProxyResult, ingest, path=["response"])

    @parametrize
    def test_streaming_response_units(self, client: Payi) -> None:
        with client.ingest.with_streaming_response.units(
            category="x",
            input=0,
            output=0,
            resource="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ingest = response.parse()
            assert_matches_type(ProxyResult, ingest, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncIngest:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_units(self, async_client: AsyncPayi) -> None:
        ingest = await async_client.ingest.units(
            category="x",
            input=0,
            output=0,
            resource="x",
        )
        assert_matches_type(ProxyResult, ingest, path=["response"])

    @parametrize
    async def test_method_units_with_all_params(self, async_client: AsyncPayi) -> None:
        ingest = await async_client.ingest.units(
            category="x",
            input=0,
            output=0,
            resource="x",
            event_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
            budget_ids=["budgetId1", "budgetId_2"],
            request_tags=["requestTag1", "request_tag_2"],
        )
        assert_matches_type(ProxyResult, ingest, path=["response"])

    @parametrize
    async def test_raw_response_units(self, async_client: AsyncPayi) -> None:
        response = await async_client.ingest.with_raw_response.units(
            category="x",
            input=0,
            output=0,
            resource="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ingest = await response.parse()
        assert_matches_type(ProxyResult, ingest, path=["response"])

    @parametrize
    async def test_streaming_response_units(self, async_client: AsyncPayi) -> None:
        async with async_client.ingest.with_streaming_response.units(
            category="x",
            input=0,
            output=0,
            resource="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ingest = await response.parse()
            assert_matches_type(ProxyResult, ingest, path=["response"])

        assert cast(Any, response.is_closed) is True
